

proc framelabel { w text } {
    frame $w -bd 2 -relief groove
    set i 0
    set parent [winfo parent $w]
    if { $parent == "." } { set parent "" }
    set label $parent.__label$i
    while { [winfo exists $label] } {
	incr i
	set label $parent.__label$i
    }
    label $label -text $text -pady 0
    place $label -in $w -x 10 -y -1 -anchor w -bordermode outside
    after idle raise $label
    return $w
}

namespace eval Pmesh {
    variable tdynmodel ""
    variable FontHeight
    variable FramesSeparation
    variable status ""
    variable ramdir

}

proc Pmesh::Main { dir { w .gid.cfd } } {
    variable tdynmodel
    variable FontHeight
    variable FramesSeparation
    variable status
    variable ramdir

    set ramdir $dir

    catch { destroy $w }
    
    set status ""
    trace var tdynmodel w "set Pmesh::status {} ;#"

    InitWindow $w "Pressures from CFD" PreCFDWindowGeom "" "" 1

    set FontHeight [font metric BoldFont -linespace ]
    set FramesSeparation [expr $FontHeight/2+1]

    framelabel $w.f1 "tdyn"
    label $w.f1.l -text [join [list "This command transfer pressures from a fluid dynamic analysis "\
	    "previusly made with program Tdyn 3D. The process is the following:\n "\
	    "\t1- Execute the Tdyn 3D analysis in another model\n"\
	    "\t2- Assign condition 'Global shell load' in Rambshell to\n\t   all required surfaces\n"\
	    "\t\tNote: The values of the assigned pressures\n\t\t      are lost\n"\
	    "\t3- Generate the mesh\n"\
	    "\t4- Enter in this window the Tdyn model name\n"\
	    "\t5- Press button 'Transfer'\n"\
	    "\t6- All 'Global shell loads' in current loadcase will be\n\t   modified for mesh\n"\
	    "\t7- Conditions on geometry are not changed"] ""] \
	    -wraplength 400 -justify left

    label $w.f1.l2 -text "Tdyn 3d model:"
    entry $w.f1.e -textvar Pmesh::tdynmodel
    button $w.f1.b -image [imagefrom openfold.gif] -command "Pmesh::LoadTdynName $w"

    grid $w.f1.l -row 1 -column 1 -columnspan 3 -sticky ew -pady $FramesSeparation
    grid $w.f1.l2 -row 2 -column 1 -sticky e
    grid $w.f1.e -row 2 -column 2 -sticky ew
    grid $w.f1.b -row 2 -column 3 -sticky w -padx 2 -pady 2
    grid columnconf $w.f1 2 -weight 1

    framelabel $w.f2 status
    label $w.f2.l -textvar Pmesh::status -justify left -anchor w
    pack $w.f2.l -fill x -expand 1 -pady $FramesSeparation -padx 5

    set color_fondo [ CCGetRGB $w [ $w cget -background]]

    frame $w.buts -bd 0 -bg [CCColorActivo $color_fondo]
    button $w.buts.b1 -text Transfer -width 8 -und 0 -command "Pmesh::Transfer $w"
    button $w.buts.b2 -text Close -width 8 -und 0 -command "destroy $w"

    grid $w.buts.b1 -row 1 -column 1 -sticky e -padx 2 -pady 1
    grid $w.buts.b2 -row 1 -column 2 -sticky w -padx 2
    grid columnconf $w.buts 1 -weight 1
    grid columnconf $w.buts 2 -weight 1

    focus $w.f1.e
    bind $w <Alt-t>  "tkButtonInvoke $w.buts.b1 ; break"
    bind $w <Alt-c>  "tkButtonInvoke $w.buts.b2 ; break"
    bind $w <Escape>  "tkButtonInvoke $w.buts.b2  ; break"
    bind $w <Return>  "tkButtonInvoke $w.buts.b1  ; break"


    grid $w.f1 -column 1 -row 1 -sticky ew -pady $FramesSeparation
    grid $w.f2 -column 1 -row 2 -sticky ew -pady [expr $FramesSeparation-4]
    grid $w.buts -column 1 -row 3 -sticky ewns
    grid columnconf $w 1 -weight 1
    grid rowconf $w 3 -weight 1

    # for testing
    bind all <Control-x><Control-l> {
	catch {
	    source {c:/Users/ramsan/barres/rambshell.gid/pmesh.tcl}
	} err
	WarnWin "pmesh code reload ($err)"
    }
}

proc Pmesh::LoadTdynName { w } {
    variable tdynmodel
    set tdynmodel [Browser-ramR project read $w {Tdyn 3D model} $tdynmodel]
}

proc Pmesh::Transfer { w } {
    global ProblemTypePriv
    variable tdynmodel
    variable ramdir
    variable status

    if { [string trim $tdynmodel] == "" } {
	WarnWin "Enter the name of a Tdyn 3D model in the entry"
	return
    }
    if { ![string match *.gid $tdynmodel] } {
	WarnWin "Tdyn model name must have extension: .gid"
	return
    }
    if { ![file isdir $tdynmodel] } {
	WarnWin "Tdyn model not found"
	return
    }
    set tdynbasename [file join $tdynmodel [file root [file tail $tdynmodel]].flavia.ram]
    if { ![file exist $tdynbasename.msh] || ![file exist $tdynbasename.res] } {
	WarnWin "Tdyn model not correct. Are you sure that this is a Tdyn 3D model?"
	return
    }

    set projectdir [ConcretePrefs::GiveProjectDir]
    if { $projectdir == "" } {
	WarnWin "Model must have a name before proceeding. Save model to get it"
	return
    }

    $w conf -cursor watch
    set status "Sending data to interpolator..."
    update

    set olddir [pwd]
    cd $projectdir
    
    set basfile [open tmpfile.bas w]
    puts $basfile "*loop intervals"
    set intervalnum [expr $ProblemTypePriv(currentloadcase)+1]
    puts $basfile "*if(LoopVar==$intervalnum)"
    puts $basfile "*set cond Global_Shell_Load *elems"
    puts $basfile "*if(CondNumEntities==0)"
    puts $basfile [join [list "error There are no elements with condition 'Global Shell Load' "\
	    "in current Loadcase\n*break"] ""]
    puts $basfile "*endif"
    puts $basfile "mesh dimension = 3 elemtype Triangle nnode = 3"
    puts $basfile "coordinates"
    puts $basfile "*loop nodes"
    puts $basfile "*NodesNum *NodesCoord(1) *NodesCoord(2) *NodesCoord(3)"
    puts $basfile "*end"
    puts $basfile "end coordinates\nelements"
    puts $basfile "*loop elems *OnlyInCond"
    puts $basfile "*elemsnum *elemsConec"
    puts $basfile "*end elems"
    puts $basfile "end elements"
    puts $basfile "*endif"
    puts $basfile "*end intervals"
    close $basfile

    catch { file delete tmpfile.mesh }
    catch { file delete tmpfile.out }

    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    .central.s disable graphics 1
    .central.s disable warnline 1
    .central.s process escape escape escape escape Files WriteForBAS tmpfile.bas \
	    tmpfile.mesh
    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

    set errormess ""
    if { ![file exists tmpfile.mesh] } { set errormess "Error writing file to disk" }
    if { $errormess == "" } {
	if { [file size tmpfile.mesh] < 100 } {
	    set fin [open tmpfile.mesh r]
	    gets $fin aa
	    if { [string match error* $aa] } {
		set errormess [lrange $aa 1 end]
	    }
	    close $fin
	}
    }
    if { $errormess != "" } {
	WarnWin $errormess
	file delete tmpfile.bas
	cd $olddir
	$w conf -cursor ""
	set status ""
	return
    }

    set status "interpolating..."
    update
    set retval [catch {
	  exec [file join $ramdir rambshell.exe] -t [file native $tdynbasename.msh] \
		  [file native $tdynbasename.res] tmpfile.mesh tmpfile.out
    } err]
    set errormess ""
   if { $retval } {
       set errormess "There have been problems in the execution ($err)"
   }
   if { $errormess == "" } {
       if { ![file readable tmpfile.out] || [file size tmpfile.out] == 0 } {
	   set errormess "There have been problems in the execution (file void)"
       }
   }
   if { $errormess != "" } {
       file delete tmpfile.bas tmpfile.mesh 
       catch { file delete tmpfile.out }
       cd $olddir
       $w conf -cursor ""
       set status ""
       WarnWin $errormess
       return
   }
   set status "retrieving data..."
   update

   .central.s process escape escape escape escape
    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
   .central.s disable graphics 1
   .central.s disable warnline 1
   
   .central.s process data intervals changeinterval \
	    [expr $ProblemTypePriv(currentloadcase)+1] escape

   set usingnow [lindex [.central.s info Project] 4]
   if { $usingnow == "GEOMETRYUSE" } {
       .central.s process escape escape escape escape Meshing MeshView
   }
   
   .central.s process escape escape escape escape Data Conditions AssignCond \
	   Global_Shell_Load 
   set fin [open tmpfile.out r]
   set inum 0
   while { ![eof $fin] } {
       gets $fin aa
       set num ""
       foreach "num val1 val2 val3" $aa ""
       if { $num != "" } {
	   .central.s process Change N-m-kg $val1 $val2 $val3 $num
	   incr inum
       }
   }
   .central.s process escape escape escape
   if { $usingnow == "GEOMETRYUSE" } {
       .central.s process escape escape escape escape Meshing MeshView
   }
   .central.s process escape escape escape escape data intervals changeinterval \
	    1 escape
   if { !$DisableGraphics } { .central.s disable graphics 0 }
   if { !$DisableWarnLine } { .central.s disable warnline 0 }

   close $fin
   file delete tmpfile.bas tmpfile.mesh
   catch { file delete tmpfile.out }
   cd $olddir
   $w conf -cursor ""
   set status "Finished. Transferred pressures for $inum mesh elements"
   WarnWin "Finished. Transferred pressures for $inum mesh elements"
}

#Pmesh::Main .cfd