namespace eval sections {
    variable sectiontype ""
    variable can
    variable width ""
    variable height ""
    variable topweight ""
    variable bottomweight ""
    variable radius ""
    variable thickness ""
    variable wingsthick ""
    variable thick
    variable area 0.0
    variable izz 0.0
    variable iyy 0.0
    variable iyz 0.0
    variable j 0.0
    variable oy 0.0
    variable oz 0.0
    variable yscale
    variable zscale
    variable cz
    variable pi 3.14159265358979323846
    variable young ""
    variable mass ""
    variable shear ""
    variable matname "steel"
    variable units N-m-kg
    variable unitshow "N/m\u00b2"
    variable unitmass  "N/m\u00b3"
    variable section_data {
	R0lGODlhGAAYAKEAADMzMwAAAL6NkMzMzCH+Dk1hZGUgd2l0aCBHSU1QACH5
	BAEKAAMALAAAAAAYABgAAAJJnI+py+3fgJwy0KuA2Lx7AWTfyIXJRWKO1EnQ
	Wb6LJgSyCN5wrh+02Rv8goYhMbAB9mi8G6uknFFSFBzpExVeR9nLxOJ1EceP
	AgA7 
    }
    variable section [image create photo -data [set section_data]]
    variable bcalculate
    
}
proc sections::ComunicateWithGiDs { op args } {
    variable area 
    variable izz 
    variable iyy 
    variable iyz 
    variable young
    variable mass
    variable shear
    variable matname
    variable units

    variable can
    #variable sectiontype ""    
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]
	    set bshape [Button $PARENT.bshape -justify center \
		    -command "sections::initwindowsections [list $GDN $STRUCT]" \
		    -image $sections::section -width 50 \
		    -helptext "Defines the shape of the section" ]
	    grid $bshape -row $ROW -column 1 -sticky nw -pady 3 -padx 2
	    upvar \#0 $GDN GidData
	    return ""
	}
	 "SYNC" {
	     set GDN [lindex $args 0]
	     set STRUCT [lindex $args 1]
	     return ""
	}
    }
}
proc sections::initwindowsections { GDN STRUCT } {
    variable can
    variable yscale
    variable zscale
    variable sectiontype 
    variable matname
	
    InitWindow .t "Sections"  PreLoadCasesWindowGeom LoadCaseWindow
    focus .t        
    grid columnconf .t 0 -weight 1
    grid rowconf .t 0 -weight 1
     ##############################################################
    #creating the notebook to organize the information           #
    ##############################################################
    set nb [NoteBook .t.notebook ]
    set page1 [$nb insert end page1 -text "Geometrical Properties"]
    set page2 [$nb insert end page2 -text "Mechanical Properties"]
    $nb raise page1
    grid $nb -sticky nsew
    grid columnconf $page1  0 -weight 1
    grid rowconf $page1 0 -weight 1
    grid columnconf $page2  0 -weight 1
    grid rowconf $page2 2 -weight 1
    set wingid [TitleFrame $page1.w -relief groove -bd 2 \
	    -text "Sections" -side left]
    set fwingid [$wingid getframe]
    grid $wingid -sticky nsew
    grid columnconf $fwingid 0 -weight 1
    grid rowconf $fwingid 0 -weight 1
    set pw [PanedWindow $fwingid.pw -side top ]
    set pane1 [$pw add -weight 1]
    set lsection [label $pane1.lsection -text Section -justify center]
    set listsection [list "Rectangular Solid" "Trapezoidal Solid" \
	    "Circular Solid" "SemiCircular Solid" "Tube" "Double T"]
    set cbsection [ComboBox $pane1.cbsection \
	    -textvariable sections::sectiontype \
	    -editable no -values $listsection]
    set title1 [TitleFrame $pane1.draw -relief groove -bd 2 \
	    -text "Visual Description" -side left]
    set f1 [$title1 getframe]
    set can [canvas $f1.can -relief raised -bd 2 -width 200\
	    -height 150 -bg white]
    set pane2 [$pw add -weight 0]
    set title2 [TitleFrame $pane2.prop -relief groove -bd 2 \
	    -text "Section Properties" -side left]
    set f2 [$title2 getframe]
    set command "sections::type $title2 $pane2 ;#"
    trace variable sections::sectiontype w $command
    bind $cbsection <Destroy> [list trace vdelete sections::sectiontype w $command]
    set message [ label $f2.canmessage -text "Select a section" -width 15 \
	    -height 5]
    set title3 [TitleFrame $pane2.coord -relief groove -bd 2 \
	    -text "Origin of Coordinates" -side left -ipad 8]
    set f3 [$title3 getframe] 
    set lycenter [label $f3.lycenter -text "Y center" -justify left]
    set eycenter [Entry $f3.eycenter -textvariable sections::oy \
	    -bd 2 -relief sunken]
    set abyscaleup [ArrowButton $f3.abyscaleup -type arrow -dir right \
	    -helptext "Increase scale range" -armcommand "sections::increasescale y"]
   set abyscaledown [ArrowButton $f3.abyscaledown -type arrow -dir left \
	    -helptext "Decrease scale range" -armcommand "sections::decreasescale y"] 
    set yscale [scale $f3.ycscale -showvalue 0 -orient horizontal \
	    -command "sections::refresh;#" \
	    -width 10 -sliderlength 10 -from -100 -to 100 \
	    -variable sections::oy -digits 4 \
	    -tickinterval 50.0 -resolution 0.5 -length 100]
    set command "sections::linkenrtyscale y ;#"
    trace variable sections::oy w $command
    bind $eycenter <Destroy> [list trace vdelete sections::oy w $command]
    set lzcenter [label $f3.lzcenter -text "Z center" -justify left]
    set ezcenter [Entry $f3.ezcenter -textvariable sections::oz \
	    -bd 2 -relief sunken]
     set abzscaleup [ArrowButton $f3.abzscaleup -type arrow -dir right \
	    -helptext "Increase scale range" -armcommand "sections::increasescale z"]
   set abzscaledown [ArrowButton $f3.abzscaledown -type arrow -dir left \
	    -helptext "Decrease scale range" -armcommand "sections::decreasescale z"] 
    set zscale [scale $f3.zcscale -showvalue 0 -orient horizontal \
	    -command "sections::refresh;#" \
	    -width 10 -sliderlength 10 -from -100 -to 100 \
	    -variable sections::oz -digits 4 \
	    -tickinterval 50.0 -resolution 0.5 -length 100]
     set command "sections::linkenrtyscale z ;#"
    trace variable sections::oz w $command
    bind $ezcenter <Destroy> [list trace vdelete sections::oz w $command]
    set bcentroid [button $f3.bcentroid -text Centroid -underline 1 \
	    -command "set sections::oy 0.0 ; set sections::oz 0.0 ; sections::refresh;#"]
    set title4 [TitleFrame $pane1.inertia -relief groove -bd 2 \
	    -text "Moments of Inertia" -side left -ipad 8]
    set f4 [$title4 getframe]
    set larea [label $f4.larea -text Area -justify left]
    set earea [Entry $f4.earea -textvariable sections::area -editable no -relief groove \
	    -bd 2 -bg lightyellow -justify right]
    set liyy [label $f4.liyy -text Iyy -justify left]
    set eiyy [Entry $f4.eiyy -textvariable sections::iyy -editable no -relief groove -bd 2 \
	    -bg lightyellow -justify right]
    set lizz [label $f4.lizz -text Izz -justify left]
    set eizz [Entry $f4.eizz -textvariable sections::izz -editable no -relief groove -bd 2  \
	    -bg lightyellow -justify right]
    set liyz [label $f4.liyz -text Iyz -justify left]
    set eiyz [Entry $f4.eiyz -textvariable sections::iyz -editable no -relief groove -bd 2  \
	    -bg lightyellow -justify right]
    set lj [label $f4.lj -text "  J" -justify left]
    set ej [Entry $f4.ej -textvariable sections::j -editable no -relief groove -bd 2  \
	    -bg lightyellow -justify right]
    set fbuttons [frame .t.fbuttons -bd 1 -relief groove ]
    set baccept [button $fbuttons.baccept -text Accept -width 20 \
	    -command "sections::accepts [list $GDN $STRUCT]"]
    set bcancel [button $fbuttons.bcancel -text Cancel -width 20 \
	    -command "sections::cancel [list $GDN $STRUCT]"]
    grid $pw -column 0 -row 0 -sticky nsew
    grid columnconf $pane1 1 -weight 1
    grid rowconf $pane1 1 -weight 1
    grid $lsection -column 0 -row 0 -sticky we
    grid $cbsection -column 1 -row 0 -sticky we -pady 4 -padx 3
    grid $title1 -column 0 -row 1 -sticky nsew -columnspan 2 -padx 3
    grid columnconf $f1 0 -weight 1
    grid rowconf $f1 0 -weight 1
    grid $can -column 0 -row 0 -sticky nsew
    grid columnconf $pane2 0 -weight 1
    grid rowconf $pane2 4 -weight 1
    if { $sectiontype == "" } {
	grid $title2 -column 0 -row 0 -sticky nsew -padx 3
	grid columnconf $f2 0 -weight 1
	grid rowconf $f2 0 -weight 1
	grid $message -column 0 -row 0 
    } else {
	sections::type  $title2 $pane2 
    }
    grid $title3 -column 0 -row 1 -sticky nsew -padx 3
    grid columnconf $f3 1 -weight 1
    grid $lycenter -column 0 -row 0 -sticky ew
    grid $eycenter -column 1 -row 0 -sticky ew -padx 4
    grid $abyscaledown -column 2 -row 0 -sticky e
    grid $abyscaleup -column 3 -row 0 -sticky w
    grid $yscale -column 0 -row 1 -sticky ew -columnspan 4 
    grid $lzcenter -column 0 -row 2 -sticky ew
    grid $ezcenter -column 1 -row 2 -sticky ew -padx 4
    grid $abzscaledown -column 2 -row 2 -sticky e
    grid $abzscaleup -column 3 -row 2 -sticky w
    grid $zscale -column 0 -row 3 -sticky ew -columnspan 4
    grid $bcentroid -column 1 -row 4 -sticky w -padx 40
    grid $title4 -column 0 -row 2 -sticky sew -columnspan 2 -padx 3
    grid columnconf $f4 1 -weight 1
    grid $larea -column 0 -row 0 -sticky ew -pady 2
    grid $earea -column 1 -row 0 -sticky ew -pady 2
    grid $liyy -column 0 -row 1 -sticky ew -pady 2
    grid $eiyy -column 1 -row 1 -sticky ew -pady 2
    grid $lizz -column 0 -row 2 -sticky ew -pady 2
    grid $eizz -column 1 -row 2 -sticky ew -pady 2
    grid $liyz -column 0 -row 3 -sticky ew -pady 2
    grid $eiyz -column 1 -row 3 -sticky ew -pady 2
    grid $lj -column 0 -row 4 -sticky ew -pady 2
    grid $ej -column 1 -row 4 -sticky ew -pady 2
    bind $can <Configure> "sections::refresh"
    grid $fbuttons -column 0 -row 1 -sticky nsew -pady 8
    grid $baccept -column 0 -row 0 -sticky nsew -padx 20 -pady 4
    grid $bcancel -column 1 -row 0 -sticky nsew -pady 4
     $nb compute_size


     set mat [TitleFrame $page2.mat -relief groove -bd 2 \
	    -text "Material" -side left -ipad 8]
    set fmat [$mat getframe]
    set lunits [label $fmat.lunits -text Units -justify left]
    set cbunits [ComboBox $fmat.cbunits -textvariable sections::units -editable no -relief groove \
	    -bd 2 -justify right -values [list N-m-kg N-cm-kg N-mm-kg Kp-cm-utm] -modifycmd sections::matproperties]
#     set command "sections::matproperties ;#"
#     trace variable sections::units w $command
#     bind $cbunits <Destroy> [list trace vdelete sections::units w $command]
    set lyoung [label $fmat.lyoung -text E -justify left]
    set eyoung [Entry $fmat.earea -textvariable sections::young -relief groove \
	    -bd 2 -justify left]
    set lyunit [label $fmat.lyunit -textvariable sections::unitshow -justify left -width 7]
    set lshear [label $fmat.lshear -text G -justify left]
    set eshear [Entry $fmat.eshear -textvariable sections::shear -relief groove \
	    -bd 2 -justify left]
    set lsunit [label $fmat.lsunit -textvariable sections::unitshow -justify left -width 7]
    set lmass [label $fmat.lmass -text "Specific Weight" -justify left]
    set emass [Entry $fmat.emass -textvariable sections::mass -relief groove \
	    -bd 2 -justify left]
    set lmunit [label $fmat.lmunit -textvariable sections::unitmass -justify left -width 7]
    set listmat [TitleFrame $page2.listmat -relief groove -bd 2 \
	    -text "Predefined Material" -side left -ipad 8]
    set flist [$listmat getframe]
    set lname [label $flist.lname -text Name -justify left]
    set materials [list steel titanium stainless]
    set cbname [ComboBox $flist.cbname -textvariable sections::matname -editable no -relief groove \
	    -bd 2 -justify right -values $materials -modifycmd sections::matproperties]
#     set command "sections::matproperties ;#"
#     trace variable sections::matname w $command
#     bind $cbname <Destroy> [list trace vdelete sections::matname w $command]
    grid $mat -column 0 -row 0 -sticky nsew
    grid $lunits -column 0 -row 0 -sticky ne
    grid $cbunits -column 1 -row 0 -sticky nsew
    grid $lyoung -column 0 -row 1 -sticky ne
    grid $eyoung -column 1 -row 1 -sticky nsew
    grid $lyunit -column 2 -row 1 -sticky ne 
    grid $lshear -column 0 -row 2 -sticky ne
    grid $eshear -column 1 -row 2 -sticky nsew
    grid $lsunit -column 2 -row 2 -sticky ne
    grid $lmass -column 0 -row 3 -sticky ne
    grid $emass -column 1 -row 3 -sticky nsew
    grid $lmunit -column 2 -row 3 -sticky ne
    grid $listmat -column 0 -row 1 -sticky nsew
    grid $lname -column 0 -row 0 -sticky ne 
    grid $cbname -column 1 -row 0 -sticky nsew
    $nb compute_size
    set matname "steel"
    sections::matproperties  
}
proc sections::type { window parent  } {
    variable sectiontype
    variable area ""
    variable izz ""
    variable iyy ""
    variable iyz ""
    variable j ""
    variable oy
    variable oz 
   
#     set oy 0.0
#     set oz 0.0
    destroy $window
    switch $sectiontype {
	"Rectangular Solid" { sections::rect_sol  $parent }
	"Trapezoidal Solid" { sections::trap_sol $parent }
	"Circular Solid" { sections::circ_sol $parent }
	"Tube" { sections::tube $parent }
	"SemiCircular Solid" { sections::semicirc_sol $parent }
	"Double T" { sections::doublet $parent }
	default { tk_messageBox -message "Section $sectiontype doesn't exist" }    
    }
}
proc sections::rect_sol { parent } {
    variable width
    variable height
    variable can
    variable yscale
    variable zscale
    variable bcalculate

    set title2 [TitleFrame $parent.prop -relief groove -bd 2 \
	    -text "Section Properties" -side left]
    set sectionfr [$title2 getframe]
    grid $title2 -column 0 -row 0 -sticky nsew -padx 3
    grid columnconf $sectionfr 1 -weight 1
    grid rowconf $sectionfr 2 -weight 1
    set lwidth [label $sectionfr.lwidth -text Width -justify left ]
    set ewidth [entry $sectionfr.ewidth -textvariable sections::width -bd 2 -relief sunken]
    set lheight [label $sectionfr.lheight -text Height -justify left]
    set eheight [entry $sectionfr.eheight -textvariable sections::height -bd 2 -relief sunken]
    set bcalculate [button $sectionfr.bcalculate -text Calculate -underline 0 \
		-command "sections::rect_sol_cal" ]
    grid $lwidth -column 0 -row 0 -sticky nwe
    grid $ewidth -column 1 -row 0 -sticky we
    grid $lheight -column 0 -row 1 -sticky nwe
    grid $eheight -column 1 -row 1 -sticky we
    grid $bcalculate -column 0 -row 2 -pady 4 -columnspan 2
    bind $bcalculate <ButtonRelease> { set sections::values [list 5 Width $sections::width Height $sections::height];  \
	if  { ! [sections::errorcntrl $sections::values] } { 
		$sections::yscale configure \
		    -to [expr $sections::width/2.0] \
		    -from [expr -$sections::width/2.0] \
		    -tickinterval [expr -$sections::width/2.0] \
		    -resolution [expr -$sections::width/200.0] ; \
		    $sections::zscale configure -to [expr $sections::height/2.0] \
		    -from [expr -$sections::height/2.0] \
		    -tickinterval [expr -$sections::height/2.0] \
		    -resolution [expr -$sections::height/200.0] 
	    }
    }
   
    bind $ewidth <KeyPress-Return> { set sections::values [list 5 Width $sections::width Height $sections::height];  \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	event generate $sections::bcalculate <ButtonRelease>}}
    bind $eheight <KeyPress-Return> { set sections::values [list 5 Width $sections::width Height $sections::height];  \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	event generate $sections::bcalculate <ButtonRelease>}}
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set x0 [expr $x/4-10]
    set y0 [expr $y/4-10]
    set x1 [expr $x0+$x/2]
    set y1 [expr $y0+$y/2]
    $can create rectangle $x0 $y0 $x1 $y1 -fill gray85
    
}
proc sections::trap_sol { parent } {
    variable topwidth
    variable bottomwidth
    variable height
    variable can
    variable yscale
    variable zscale 
    variable bcalculate

    set title2 [TitleFrame $parent.prop -relief groove -bd 2 \
	    -text "Section Properties" -side left]
    set sectionfr [$title2 getframe]
    grid $title2 -column 0 -row 0 -sticky nsew -padx 3
    grid columnconf $sectionfr 1 -weight 1
    grid rowconf $sectionfr 4 -weight 1
    #set topwidth ""
    #set bottomwidth ""
    #set height ""
    set ltopwidth [label $sectionfr.ltopwidth -text "Top Width" -justify right]
    set etopwidth [entry $sectionfr.etopwidth -textvariable sections::topwidth -bd 2 \
	    -relief sunken]
    set lbottomwidth [label $sectionfr.lbottomwidth -text "Bottom Width" -justify right]
    set ebottomwidth [entry $sectionfr.ebottomwidth -textvariable sections::bottomwidth \
	    -bd 2 -relief sunken]
    set lheight [label $sectionfr.lheight -text Height -justify right]
    set eheight [entry $sectionfr.eheight -textvariable sections::height -bd 2 -relief sunken]
    set bcalculate [button $sectionfr.bcalculate -text Calculate -underline 0 \
		-command "sections::trap_sol_cal"]
    grid $ltopwidth -column 0 -row 0 -sticky nw
    grid $etopwidth -column 1 -row 0 -sticky we
    grid $lbottomwidth -column 0 -row 1 -sticky nw
    grid $ebottomwidth -column 1 -row 1 -sticky we
    grid $lheight -column 0 -row 2 -sticky nw
    grid $eheight -column 1 -row 2 -sticky we
    grid $bcalculate -column 0 -row 3 -pady 4 -columnspan 2
    if { $topwidth <= $bottomwidth } {
	bind $bcalculate <ButtonRelease> { set sections::values [list 7 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height];  \
	    if  { ! [sections::errorcntrl $sections::values] } {
		set sections::cz [expr ((2.0*$sections::topwidth+$sections::bottomwidth)\
		        /($sections::topwidth+$sections::bottomwidth))*($sections::height/3.0)] ; 
		$sections::yscale configure \
		-to [expr $sections::bottomwidth/2.0] \
		-from [expr -$sections::bottomwidth/2.0] \
		-tickinterval [expr -$sections::bottomwidth/2.0] \
		-resolution [expr -$sections::bottomwidth/200.0] ; \
		$sections::zscale configure -to [expr $sections::height-$sections::cz] \
		-from [expr -$sections::cz] \
		-tickinterval [expr -$sections::height/2.0] \
		-resolution [expr -$sections::height/200.0] 
	}
    }
    } else {
	bind $bcalculate <ButtonRelease> { set sections::values [list 7 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height];  \
	    if  { ! [sections::errorcntrl $sections::values] } {
		set sections::cz [expr ((2*$sections::topwidth+$sections::bottomwidth) \
		        /($sections::topwidth+$sections::bottomwidth))*($sections::height/3.0)] ; 
		$sections::yscale configure \
		  -to [expr $sections::topwidth/2.0] \
		 -from [expr -$sections::topwidth/2.0] \
		-tickinterval [expr -$sections::topwidth/2.0] \
		-resolution [expr -$sections::topwidth/200.0] ; \
		$sections::zscale configure -to [expr $sections::height-$sections::cz] \
		-from [expr -$sections::cz] \
		-tickinterval [expr -$sections::height/2.0] \
		-resolution [expr -$sections::height/200.0] }
	}
    }
    bind $etopwidth <KeyPress-Return> { set sections::values [list 7 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height];  \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    bind $ebottomwidth <KeyPress-Return> { set sections::values [list 7 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height];  \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    bind $eheight <KeyPress-Return> { set sections::values [list 7 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height];  \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set xmarge [expr $x*0.1]
    set ymarge [expr $y*0.1]
    set a1 [expr $xmarge+($x-2*$xmarge-140)/2]
    set a2 [expr $y/2+100/2]
    set b1 [expr $xmarge+($x-2*$xmarge-100)/2]
    set b2 [expr $y/2-100/2]
    set c1 [expr $b1+100]
    set c2 [expr $y/2-100/2]
    set d1 [expr $a1+140]
    set d2 [expr $y/2+100/2]
    set vertexs "$a1 $a2 $b1 $b2 $c1 $c2 $d1 $d2"
    $can create polygon $vertexs -fill gray85 -outline black
}
proc sections::circ_sol { parent } {
    variable radius
    variable can
    variable yscale
    variable zscale
    variable cz
    variable bcalculate

    set title2 [TitleFrame $parent.prop -relief groove -bd 2 \
	    -text "Section Properties" -side left]
    set sectionfr [$title2 getframe]
    grid $title2 -column 0 -row 0 -sticky nsew
    grid columnconf $sectionfr 1 -weight 1
    grid rowconf $sectionfr 2 -weight 1
    #set radius ""
    set lradius [label $sectionfr.lradius -text Radius -justify left]
    set eradius [entry $sectionfr.eradius -textvariable sections::radius -bd 2 -relief sunken]
    set bcalculate [button $sectionfr.bcalculate -text Calculate -underline 0 \
		-command "sections::circ_sol_cal" ]
    grid $lradius -column 0 -row 0 -sticky nwe
    grid $eradius -column 1 -row 0 -sticky we
    grid $bcalculate -column 0 -row 2 -pady 4 -columnspan 2
    bind $bcalculate <ButtonRelease> { set sections::values [list 3 Radius $sections::radius];  \
	    if  { ! [sections::errorcntrl $sections::values] } { 
	    set sections::cz $sections::radius] ; 
	    $sections::yscale configure \
	     -to [expr $sections::radius] \
	    -from [expr -$sections::radius] \
	    -tickinterval [expr -$sections::radius] \
	    -resolution [expr -$sections::radius/200.0] ; \
	    $sections::zscale configure -to [expr $sections::radius] \
	    -from [expr -$sections::radius] \
	    -tickinterval [expr -$sections::radius/2.0] \
	    -resolution [expr -$sections::radius/200.0]
	    }
    }
    bind $eradius <KeyPress-Return>  { set sections::values [list 3 Radius $sections::radius];  \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set x0 [expr $x/2-60]
    set y0 [expr ($y/2-60)]
    set x1 [expr $x0+2*60]
    set y1 [expr $y0+2*60]
    $can create oval $x0 $y0 $x1 $y1 -fill gray85 
}

proc sections::tube { parent } {
    variable radius
    variable thickness
    variable can
    variable yscale
    variable zscale
    variable cz
    variable bcalculate

    set title2 [TitleFrame $parent.prop -relief groove -bd 2 \
	    -text "Section Properties" -side left]
    set sectionfr [$title2 getframe]
    grid $title2 -column 0 -row 0 -sticky nsew
    grid columnconf $sectionfr 1 -weight 1
    grid rowconf $sectionfr 2 -weight 1
    #set radius ""
    set lradius [label $sectionfr.lradius -text Radius -justify left]
    set eradius [entry $sectionfr.eradius -textvariable sections::radius -bd 2 -relief sunken]
    set lthick [label $sectionfr.lthick -text Thickness -justify left]
    set ethick [entry $sectionfr.ethick -textvariable sections::thickness -bd 2 -relief sunken]
    set bcalculate [button $sectionfr.bcalculate -text Calculate -underline 0 \
		-command "sections::tube_cal" ]
    grid $lradius -column 0 -row 0 -sticky nwe
    grid $eradius -column 1 -row 0 -sticky we
    grid $lthick -column 0 -row 1 -sticky nwe
    grid $ethick -column 1 -row 1 -sticky we
    grid $bcalculate -column 0 -row 2 -pady 4 -columnspan 2
    bind $bcalculate <ButtonRelease> {
	set sections::values [list "" Radius $sections::radius Thickness $sections::thickness]
	if  { ! [sections::errorcntrl $sections::values] } { 
	    set sections::cz $sections::radius] ; 
	    $sections::yscale configure \
		-to [expr $sections::radius] \
		-from [expr -$sections::radius] \
		-tickinterval [expr -$sections::radius] \
		-resolution [expr -$sections::radius/200.0] ; \
		$sections::zscale configure -to [expr $sections::radius] \
		-from [expr -$sections::radius] \
		-tickinterval [expr -$sections::radius/2.0] \
		-resolution [expr -$sections::radius/200.0]
	}
    }
    bind $eradius <KeyPress-Return>  {
	set sections::values [list "" Radius $sections::radius Thickness $sections::thickness]
	if  { ! [sections::errorcntrl $sections::values] } {
	    tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>
	}
    }
    bind $ethick <KeyPress-Return> [bind $eradius <KeyPress-Return>]
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set x0 [expr $x/2-60]
    set y0 [expr ($y/2-60)]
    set x1 [expr $x0+2*60]
    set y1 [expr $y0+2*60]
    $can create oval $x0 $y0 $x1 $y1 -fill gray85 
    set x0_i [expr $x/2-55]
    set y0_i [expr ($y/2-55)]
    $can create oval $x0_i $y0_i [expr $x0_i+2*55] [expr $y0_i+2*55] \
	-fill [$can cget -background] 
}

proc sections::semicirc_sol { parent } {
    variable radius
    variable can
    variable yscale
    variable zscale
    variable cz
    variable bcalculate

    set title2 [TitleFrame $parent.prop -relief groove -bd 2 \
	    -text "Section Properties" -side left]
    set sectionfr [$title2 getframe]
    grid $title2 -column 0 -row 0 -sticky nsew -padx 3
    grid columnconf $sectionfr 1 -weight 1
    grid rowconf $sectionfr 2 -weight 1
    #set radius ""
    set lradius [label $sectionfr.lradius -text Radius -justify left]
    set eradius [entry $sectionfr.eradius -textvariable sections::radius -bd 2 -relief sunken]
    set bcalculate [button $sectionfr.bcalculate -text Calculate -underline 0 \
		-command "sections::semicirc_sol_cal" ]
    grid $lradius -column 0 -row 0 -sticky nwe
    grid $eradius -column 1 -row 0 -sticky we
    grid $bcalculate -column 0 -row 2 -pady 4 -columnspan 2
    bind $bcalculate <ButtonRelease> { set sections::values [list 3 Radius $sections::radius] ;  \
	    if  { ! [sections::errorcntrl $sections::values] } { 
	    set sections::cz [expr 4*$sections::radius/(3*$sections::pi)] ; 
	    $sections::yscale configure \
	     -to [expr $sections::radius] \
	    -from [expr -$sections::radius] \
	    -tickinterval [expr -$sections::radius] \
	    -resolution [expr -$sections::radius/200.0] ; \
	    $sections::zscale configure -to [expr $sections::radius-$sections::cz] \
	    -from [expr -$sections::cz] \
	    -tickinterval [expr -$sections::radius/2.0] \
	    -resolution [expr -$sections::radius/200.0]
	    }
    }
    bind $eradius <KeyPress-Return>  { set sections::values [list 3 Radius $sections::radius];  \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set x0 [expr $x/2-75]
    set y0 [expr ($y/2-75)+25]
    set x1 [expr $x0+2*75]
    set y1 [expr $y0+2*75]
    $can create arc $x0 $y0 $x1 $y1 -fill gray85 -extent 180 -start 0
}
proc sections::doublet { parent } {
    variable topwidth
    variable bottomwidth
    variable height
    variable wingsthick
    variable thick
    variable can
    variable yscale
    variable zscale
    variable cz
    variable bcalculate

    set title2 [TitleFrame $parent.prop -relief groove -bd 2 \
	    -text "Section Properties" -side left]
    set sectionfr [$title2 getframe]
    grid $title2 -column 0 -row 0 -sticky nsew -padx 3
    grid columnconf $sectionfr 1 -weight 1
    grid rowconf $sectionfr 5 -weight 1
    #set height ""
    #set topwidth ""
    #set bottomwidth ""
    #set wingsthick ""
    #set thick ""
    set ltopwidth [label $sectionfr.ltopwidth -text "Top Width" -justify left]
    set etopwidth [entry $sectionfr.etopwidth \
	    -textvariable sections::topwidth -bd 2 -relief sunken]
    set lbottomwidth [label $sectionfr.lbottomwidth -text "Bottom Width" -justify left]
    set ebottomwidth [entry $sectionfr.ebottomwidth \
	    -textvariable sections::bottomwidth -bd 2 -relief sunken]
    set lheight [label $sectionfr.lheight -text Height -justify left]
    set eheight [entry $sectionfr.eheight -textvariable sections::height -bd 2 -relief sunken]
    set lwingsthick [label $sectionfr.lwingsthick -text "Wings Thick." -justify left]
    set ewingsthick [entry $sectionfr.ewingsthick \
	    -textvariable sections::wingsthick -bd 2 -relief sunken]
    set lthick [label $sectionfr.lthick -text "Central Thick." -justify left]
    set ethick [entry $sectionfr.ethick \
	    -textvariable sections::thick -bd 2 -relief sunken]
    set bcalculate [button $sectionfr.bcalculate -text Calculate -underline 0 \
		-command "sections::doublet_cal" ]
    grid $ltopwidth -column 0 -row 0 -sticky nw
    grid $etopwidth -column 1 -row 0 -sticky we
    grid $lbottomwidth -column 0 -row 1 -sticky nw
    grid $ebottomwidth -column 1 -row 1 -sticky we
    grid $lheight -column 0 -row 2 -sticky nw
    grid $eheight -column 1 -row 2 -sticky we
    grid $lwingsthick -column 0 -row 3 -sticky nw
    grid $ewingsthick -column 1 -row 3 -sticky we
    grid $lthick -column 0 -row 4 -sticky nw
    grid $ethick -column 1 -row 4 -sticky we
    grid $bcalculate -column 0 -row 5 -pady 4 -columnspan 2
    if { $topwidth <= $bottomwidth } {
	bind $bcalculate <ButtonRelease> { set sections::values [list 11 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height \
		WingsThick. $sections::wingsthick Thick. $sections::thick] ;  \
	    if  { ! [sections::errorcntrl $sections::values] } {
		set sections::area [expr $sections::topwidth*$sections::wingsthick+ \
		        $sections::bottomwidth*$sections::wingsthick+ \
		        ($sections::height-2*$sections::wingsthick)*$sections::thick]
		set sections::cz [expr $sections::height-(1.0/(2.0*$sections::area))* \
		        (($sections::thick*pow($sections::height,2))+ \
		        pow($sections::wingsthick,2)*($sections::topwidth-$sections::thick)+ \
		        $sections::wingsthick*($sections::bottomwidth-$sections::thick)* \
		        (2*$sections::height-$sections::wingsthick))] ; 
		$sections::yscale configure \
		 -to [expr $sections::bottomwidth/2.0] \
		-from [expr -$sections::bottomwidth/2.0] \
		-tickinterval [expr -$sections::bottomwidth/2.0] \
		-resolution [expr -$sections::bottomwidth/200.0] ; \
		$sections::zscale configure -to [expr $sections::height-$sections::cz] \
		-from [expr -$sections::cz] \
		-tickinterval [expr -$sections::height/2.0] \
		-resolution [expr -$sections::height/200.0] 
	}
      }
    } else {
	bind $bcalculate <ButtonRelease> { set sections::values [list 11 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height \
		WingsThick. $sections::wingsthick Thick. $sections::thick] ;   \
	    if  { ! [sections::errorcntrl $sections::values] } {
	    set sections::area [expr $sections::topwidth*$sections::wingsthick+ \
		    $sections::bottomwidth*$sections::wingsthick+ \
		    ($sections::height-2*$sections::wingsthick)*$sections::thick]
	     set sections::cz [expr $sections::height-(1.0/(2.0*$sections::area))* \
		     (($sections::thick*pow($sections::height,2))+ \
		    pow($sections::wingsthick,2)*($sections::topwidth-$sections::thick)+ \
		    $sections::wingsthick*($sections::bottomwidth-$sections::thick)* \
		    (2*$sections::height-$sections::wingsthick))] ; 
	    $sections::yscale configure \
		    -to [expr $sections::topwidth/2.0] \
		    -from [expr -$sections::topwidth/2.0] \
		    -tickinterval [expr -$sections::topwidth/2.0] \
		    -resolution [expr -$sections::topwidth/200.0] ; \
		    $sections::zscale configure -to [expr $sections::height-$sections::cz] \
		    -from [expr -$sections::cz] \
		    -tickinterval [expr -$sections::height/2.0] \
		    -resolution [expr -$sections::height/200.0] 
	}
      }
    }
    bind $etopwidth <KeyPress-Return>  { set sections::values [list 11 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height \
		WingsThick. $sections::wingsthick Thick. $sections::thick] ;   \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    bind $ebottomwidth <KeyPress-Return>  { set sections::values [list 11 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height \
		WingsThick. $sections::wingsthick Thick. $sections::thick] ;   \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    bind $eheight <KeyPress-Return>  { set sections::values [list 11 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height \
		WingsThick. $sections::wingsthick Thick. $sections::thick] ;   \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    bind $ewingsthick <KeyPress-Return> { set sections::values [list 11 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height \
		WingsThick. $sections::wingsthick Thick. $sections::thick] ;   \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    bind $ethick <KeyPress-Return>  { set sections::values [list 11 TopWidth $sections::topwidth \
		BottomWidth $sections::bottomwidth Height $sections::height \
		WingsThick. $sections::wingsthick Thick. $sections::thick] ;   \
	if  { ! [sections::errorcntrl $sections::values] } { tkButtonInvoke $sections::bcalculate
	    event generate $sections::bcalculate <ButtonRelease>}}
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set a1 [expr ($x-$x*0.8)/2]
    set a2 [expr ($y-$y*0.8)/2]
    set b1 [expr $a1+$x*0.8]
    set b2 $a2
    set c1 $b1
    set c2 [expr $a2+$y*0.1]
    set d1 [expr $b1-$x*0.3]
    set d2 $c2
    set e1 $d1
    set e2 [expr $d2+$y*0.6]
    set f1 [expr $e1+$x*0.3]
    set f2 $e2
    set g1 $f1
    set g2 [expr $f2+$y*0.1]
    set h1 [expr $f1-$x*0.8]
    set h2 $g2
    set i1 $h1
    set i2 [expr $h2-$y*0.1]
    set j1 [expr $i1+$x*0.3]
    set j2 $i2
    set k1 $j1
    set k2 $d2
    set l1 [expr $k1-$x*0.3]
    set l2 $k2
    set vertexs "$a1 $a2 $b1 $b2 $c1 $c2 $d1 $d2 $e1 $e2 $f1 $f2 $g1 $g2 $h1 \
	    $h2 $i1 $i2 $j1 $j2 $k1 $k2 $l1 $l2"
    $can create polygon $vertexs -fill gray85 -outline black
    
}
proc sections::refresh { } {
    variable sectiontype
    switch $sectiontype {
	"Rectangular Solid" { sections::rect_sol_cal }
	"Trapezoidal Solid" { sections::trap_sol_cal }
	"Circular Solid" { sections::circ_sol_cal }
	"Tube" { sections::tube_cal }
	"SemiCircular Solid" { sections::semicirc_sol_cal }
	"Double T" { sections::doublet_cal }
	default { }    
    }
}
proc sections::rect_sol_cal { } {
    variable can
    variable width
    variable height
    variable area
    variable izz
    variable iyy
    variable iyz
    variable oy
    variable oz
    variable j
    
  
    if { ![string is double -strict $height] || ![string is double -strict $width] } {
	return 
    }
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set xmarge [expr $x*0.1]
    set ymarge [expr $y*0.2]
    set xscale [expr ($x-2*$xmarge)/$width]
    set yscale [expr ($y-2*$ymarge)/$height]
    if { $xscale <= $yscale } {
	set xdraw [expr $xscale*$width]
	set ydraw [expr $xscale*$height]
	set oydraw [expr $xscale*$oy]
	set ozdraw [expr $xscale*$oz]
    } else {
	set xdraw [expr $yscale*$width]
	set ydraw [expr $yscale*$height]
	set oydraw [expr $yscale*$oy]
	set ozdraw [expr $yscale*$oz]
    }
    set x0 [expr $xmarge+(($x-2*$xmarge)-$xdraw)/2]
    set y0 [expr $ymarge+(($y-2*$ymarge)-$ydraw)/2]
    set x1 [expr $x0+$xdraw]
    set y1 [expr $y0+$ydraw]
    $can create rectangle $x0 $y0 $x1 $y1 -fill gray
    $can create line [expr $oydraw+$x/2] [expr -$ozdraw+$y/2] [expr $oydraw+$x/2] \
	    [expr $ymarge-15] -arrow last 
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw] [expr $x-$xmarge+15] \
	    [expr $y/2-$ozdraw] -arrow last
    $can create oval [expr $x/2-3] [expr $y/2-3] [expr $x/2+3] [expr $y/2+3] -fill red
    $can create text [expr $x/2+10+$oydraw] [expr $ymarge+10] -text Z
    $can create text [expr $x-$xmarge-10] [expr $y/2-10-$ozdraw] -text Y
    $can create text [expr $x/2-3] [expr $y/2+10] -text G
    set area [format %.6g [expr $width*$height]]
    set iyy [format %.6g [expr (($width*pow($height,3))/12.0)+pow($oz,2)*$area]]
    set izz [format %.6g [expr (($height*pow($width,3))/12.0)+pow($oy,2)*$area]]
    set iyz [format %.6g [expr abs($oz)*abs($oy)*$area]]
    set j [format %.6g [expr 0.3*pow($height,3)*pow($width,3)/(pow($height,2)+pow($width,2))]]
   
}
proc sections::trap_sol_cal { } {
    variable can
    variable topwidth
    variable bottomwidth
    variable height
    variable area 
    variable izz
    variable iyy
    variable iyz
    variable j
    variable oy
    variable oz
       
     if { [string trim $topwidth] == "" || [string trim $height] == "" || [string trim $bottomwidth] == ""} {
	return ""
    }
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set xmarge [expr $x*0.1]
    set ymarge [expr $y*0.2]
    if { $topwidth <= $bottomwidth } {
	set xscale [expr ($x-2*$xmarge)/$bottomwidth]
    } else {
	set xscale [expr ($x-2*$xmarge)/$topwidth]
    }
    set yscale [expr ($y-2*$ymarge)/$height]
    if { $xscale <= $yscale } {
	set topdraw [expr $xscale*$topwidth]
	set bottomdraw [expr $xscale*$bottomwidth]
	set ydraw [expr $xscale*$height]
	set oydraw [expr $xscale*$oy]
	set ozdraw [expr $xscale*$oz]
	set scale [expr double($xscale)]
    } else {
	set topdraw [expr $yscale*$topwidth]
	set bottomdraw [expr $yscale*$bottomwidth]
	set ydraw [expr $yscale*$height]
	set oydraw [expr $yscale*$oy]
	set ozdraw [expr $yscale*$oz]
	set scale [expr double($yscale)]
    }
    set cy [expr ((2*$topdraw+$bottomdraw)/($topdraw+$bottomdraw)) \
	    *($ydraw/3.0)]
    set a1 [expr $xmarge+($x-2*$xmarge-$bottomdraw)/2]
    set a2 [expr $y/2+$cy]
    set b1 [expr $xmarge+($x-2*$xmarge-$topdraw)/2]
    set b2 [expr $a2-$ydraw]
    set c1 [expr $b1+$topdraw]
    set c2 [expr $a2-$ydraw]
    set d1 [expr $a1+$bottomdraw]
    set d2 [expr $y/2+$cy]
    set vertexs "$a1 $a2 $b1 $b2 $c1 $c2 $d1 $d2"
    $can create polygon $vertexs -fill gray -outline black
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw] [expr $x/2+$oydraw] \
	    [expr $ymarge-10] -arrow last
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw] [expr $x-$xmarge+10] \
	    [expr $y/2-$ozdraw] -arrow last
    $can create oval [expr $x/2-3] [expr $y/2-3] [expr $x/2+3] [expr $y/2+3] -fill red
    $can create text [expr $x/2+10+$oydraw] [expr $ymarge+10] -text Z
    $can create text [expr $x-$xmarge-10] [expr $y/2-10-$ozdraw] -text Y
    $can create text [expr $x/2-3] [expr $y/2+10] -text G
    set area [format %.6g [expr ($topwidth+$bottomwidth)*$height/2.0]]
    set iyy [format %.6g [expr (pow($topwidth,2)+4*$topwidth*$bottomwidth+pow($bottomwidth,2)) \
	    *(pow($height,3)/(36*($topwidth+$bottomwidth)))+pow($oz,2)*$area]]
    set izz [format %.6g [expr ($height*($topwidth+$bottomwidth)*\
	    (pow($topwidth,2)+pow($bottomwidth,2)))/48.0+pow($oy,2)*$area]]
    set iyz [format %.6g [expr abs($oz)*abs($oy)*$area]]
    if { $topwidth <= $bottomwidth } {
	set j [format %.6g [expr 0.3*pow($height,3)*pow($topwidth,3)/(pow($height,2)+pow($topwidth,2))]]
    } else {
	set j [format %.6g [expr 0.3*pow($height,3)*pow($bottomwidth,3)/(pow($height,2)+pow($bottomwidth,2))]]
    }
    set cy [expr $cy/$scale]
    
    
}
proc sections::circ_sol_cal { } {
    variable can
    variable radius
    variable area 
    variable izz 
    variable iyy 
    variable iyz 
    variable j
    variable oy
    variable oz
    variable  pi
    
    if {![string is double -strict $radius]  } {
	return
    }
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set xmarge [expr $x*0.1]
    set ymarge [expr $y*0.2]
    set xscale [expr ($x-2*$xmarge)/(2*$radius)]
    set yscale [expr ($y-2*$ymarge)/(2*$radius)]
    if { $xscale <= $yscale } {
	set radiusdraw [expr $xscale*$radius]
	set oydraw [expr $xscale*$oy]
	set ozdraw [expr $xscale*$oz]
    } else {
	set radiusdraw [expr $yscale*$radius]
	set oydraw [expr $yscale*$oy]
	set ozdraw [expr $yscale*$oz]
    }
    set x0 [expr $x/2-$radiusdraw]
    set y0 [expr $y/2-$radiusdraw]
    set x1 [expr $x0+2*$radiusdraw]
    set y1 [expr $y0+2*$radiusdraw]
    $can create oval $x0 $y0 $x1 $y1 -fill gray 
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw] [expr $x/2+$oydraw] \
	    [expr $ymarge-10] -arrow last
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw]  [expr $x-$xmarge] \
	    [expr $y/2-$ozdraw] -arrow last
    $can create oval [expr $x/2-3] [expr $y/2-3] [expr $x/2+3] [expr $y/2+3] -fill red
    $can create text [expr $x/2-5+$oydraw] [expr $ymarge+10] -text Z
    $can create text [expr $x-$xmarge-10] [expr $y/2-10-$ozdraw] -text Y
    $can create text [expr $x/2-3] [expr $y/2+10] -text G
    set area [expr $pi*pow($radius,2)]
    set iyy [expr $pi*pow($radius,4)/4.0+pow($oz,2)*$area]
    set izz [expr $pi*pow($radius,4)/4.0+pow($oy,2)*$area]
    set iyz [expr abs($oz)*abs($oy)*$area]
    set j [expr $pi*pow($radius,4)/2.0]
    set area [format %.6g $area]
    set iyy [format %.6g $iyy]
    set izz [format %.6g $izz]
    set iyz [format %.6g $iyz]
    set j [format %.6g $j]
       
}

proc sections::tube_cal { } {
    variable can
    variable radius
    variable thickness
    variable area 
    variable izz 
    variable iyy 
    variable iyz 
    variable j
    variable oy
    variable oz
    variable  pi
    
    if {![string is double -strict $radius] || ![string is double -strict $thickness] } {
	return
    }
    $can delete all

    set r1 $radius
    set r2 [expr {$radius-$thickness}]

    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set xmarge [expr $x*0.1]
    set ymarge [expr $y*0.2]
    set xscale [expr ($x-2*$xmarge)/(2*$r1)]
    set yscale [expr ($y-2*$ymarge)/(2*$r1)]
    if { $xscale <= $yscale } {
	set scale $scale
    } else { set scale $yscale }
    
    set r1d [expr $scale*$r1]
    set r2d [expr $scale*$r2]
    set oydraw [expr $scale*$oy]
    set ozdraw [expr $scale*$oz]

    set x0 [expr $x/2-$r1d]
    set y0 [expr $y/2-$r1d]
    set x1 [expr $x0+2*$r1d]
    set y1 [expr $y0+2*$r1d]
    $can create oval $x0 $y0 $x1 $y1 -fill gray
    foreach "x0_r2 y0_r2" [list [expr {$x/2-$r2d}] [expr {$y/2-$r2d}]] break
    $can create oval $x0_r2 $y0_r2 [expr {$x0_r2+2*$r2d}] [expr {$y0_r2+2*$r2d}] \
	-fill [$can cget -background]
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw] [expr $x/2+$oydraw] \
	    [expr $ymarge-10] -arrow last
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw]  [expr $x-$xmarge] \
	    [expr $y/2-$ozdraw] -arrow last
    $can create oval [expr $x/2-3] [expr $y/2-3] [expr $x/2+3] [expr $y/2+3] -fill red
    $can create text [expr $x/2-5+$oydraw] [expr $ymarge+10] -text Z
    $can create text [expr $x-$xmarge-10] [expr $y/2-10-$ozdraw] -text Y
    $can create text [expr $x/2-3] [expr $y/2+10] -text G
    set area [expr {$pi*pow($r1,2)-$pi*pow($r2,2)}]
    set iyy [expr $pi/4.0*(pow($r1,4)-pow($r2,4))+pow($oz,2)*$area]
    set izz [expr $pi/4.0*(pow($r1,4)-pow($r2,4))+pow($oy,2)*$area]
    set iyz [expr abs($oz)*abs($oy)*$area]
    set j [expr {$pi*(pow($r1,4)-pow($r2,4))/2.0}]
    set area [format %.6g $area]
    set iyy [format %.6g $iyy]
    set izz [format %.6g $izz]
    set iyz [format %.6g $iyz]
    set j [format %.6g $j]       
}

proc sections::semicirc_sol_cal { } {
    variable can
    variable radius
    variable area 
    variable izz 
    variable iyy 
    variable iyz 
    variable oy
    variable oz
    variable  pi
    variable j
    
     if { ![string is double -strict $radius] } {
	return 
    }
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set xmarge [expr $x*0.1]
    set ymarge [expr $y*0.2]
    set xscale [expr ($x-2*$xmarge)/(2*$radius)]
    set yscale [expr ($y-2*$ymarge)/(2*$radius)]
    if { $xscale <= $yscale } {
	set radiusdraw [expr $xscale*$radius]
	set oydraw [expr $xscale*$oy]
	set ozdraw [expr $xscale*$oz]
    } else {
	set radiusdraw [expr $yscale*$radius]
	set oydraw [expr $yscale*$oy]
	set ozdraw [expr $yscale*$oz]
    }
    set x0 [expr $x/2-$radiusdraw]
    set y0 [expr $y/2-$radiusdraw+4*$radiusdraw/(3*$pi)]
    set x1 [expr $x0+2*$radiusdraw]
    set y1 [expr $y0+2*$radiusdraw]
    $can create arc $x0 $y0 $x1 $y1 -fill gray -extent 180 -start 0
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw] [expr $x/2+$oydraw] \
	    [expr $ymarge-10] -arrow last
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw]  [expr $x-$xmarge+10] \
	    [expr $y/2-$ozdraw] -arrow last
    $can create oval [expr $x/2-3] [expr $y/2-3] [expr $x/2+3] [expr $y/2+3] -fill red
    $can create text [expr $x/2-5+$oydraw] [expr $ymarge+10] -text Z
    $can create text [expr $x-$xmarge-10] [expr $y/2-10-$ozdraw] -text Y
    $can create text [expr $x/2-3] [expr $y/2+10] -text G
    set area [format %.6g [expr $pi*pow($radius,2)/2.0]]
    set iyy [format %.6g [expr $pi*pow($radius,4)/8.0]]
    set iyy [format %.6g [expr $iyy-(8/(9*$pi))*pow($radius,4)+pow($oz,2)*$area]]
    set izz [format %.6g [expr ($pi*pow($radius,4)/8.0)+pow($oy,2)*$area]]
    set iyz [format %.6g [expr abs($oz)*abs($oy)*$area]]
    set j [format %.6g [expr $pi*pow($radius,4)/4.0]]
}
proc sections::doublet_cal { } {
    variable can
    variable topwidth
    variable bottomwidth
    variable wingsthick
    variable thick
    variable height
    variable area 
    variable cz
    variable izz
    variable iyy
    variable iyz
    variable j
    variable oy
    variable oz
    
     if {![string is double -strict $height]  || ![string is double -strict $topwidth]  \
	||![string is double -strict $bottomwidth]  || ![string is double -strict $wingsthick] \
	||![string is double -strict  $thick] } {
	return 
    }
    $can delete all
    set x [ winfo width $can ]
    set y [ winfo height $can ]
    set xmarge [expr $x*0.1]
    set ymarge [expr $y*0.2]
    set h [expr $height-2*$wingsthick]
    if { $h <= 0.0 } {
	WarnWin "Thicknesses added up are greater than the section height " .t
    }
    set area [format %.6g [expr $topwidth*$wingsthick+$bottomwidth*$wingsthick+$h*$thick]]
    set cz [expr $height-(1.0/(2.0*$area))*(($thick*pow($height,2))+ \
	    pow($wingsthick,2)*($topwidth-$thick)+ \
	    $wingsthick*($bottomwidth-$thick)*(2*$height-$wingsthick))]
    if { $topwidth <= $bottomwidth } {
	set xscale [expr ($x-2*$xmarge)/$bottomwidth]
    } else {
	set xscale [expr ($x-2*$xmarge)/$topwidth]
    }
    set yscale [expr ($y-2*$ymarge)/$height]
    if { $xscale <= $yscale } {
	set topdraw [expr $xscale*$topwidth]
	set bottomdraw [expr $xscale*$bottomwidth]
	set ydraw [expr $xscale*$height]
	set wingsdraw [expr $xscale*$wingsthick]
	set thickdraw [expr $xscale*$thick]
	set hdraw [expr $xscale*$h]
	set czdraw [expr $xscale*$cz]
	set oydraw [expr $xscale*$oy]
	set ozdraw [expr $xscale*$oz]
    } else {
	set topdraw [expr $yscale*$topwidth]
	set bottomdraw [expr $yscale*$bottomwidth]
	set ydraw [expr $yscale*$height]
	set wingsdraw [expr $yscale*$wingsthick]
	set thickdraw [expr $yscale*$thick]
	set hdraw [expr $yscale*$h]
	set czdraw [expr $yscale*$cz]
	set oydraw [expr $yscale*$oy]
	set ozdraw [expr $yscale*$oz]
    }
    set a1 [expr $xmarge+($x-2*$xmarge-$topdraw)/2]
    set a2 [expr $y/2-($ydraw-$czdraw)]
    set b1 [expr $a1+$topdraw]
    set b2 $a2
    set c1 $b1
    set c2 [expr $a2+$wingsdraw]
    set d1 [expr $b1-($topdraw-$thickdraw)/2]
    set d2 $c2
    set e1 $d1
    set e2 [expr $d2+($ydraw-2*$wingsdraw)]
    set f1 [expr $e1+($bottomdraw-$thickdraw)/2]
    set f2 $e2
    set g1 $f1
    set g2 [expr $f2+$wingsdraw]
    set h1 [expr $f1-$bottomdraw]
    set h2 $g2
    set i1 $h1
    set i2 [expr $h2-$wingsdraw]
    set j1 [expr $i1+($bottomdraw-$thickdraw)/2]
    set j2 $i2
    set k1 $j1
    set k2 $d2
    set l1 [expr $k1-($topdraw-$thickdraw)/2]
    set l2 $k2
    set vertexs "$a1 $a2 $b1 $b2 $c1 $c2 $d1 $d2 $e1 $e2 $f1 $f2 $g1 $g2 $h1 \
	    $h2 $i1 $i2 $j1 $j2 $k1 $k2 $l1 $l2"
    $can create polygon $vertexs -fill gray -outline black
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw] [expr $x/2+$oydraw] \
	    [expr $ymarge-10] -arrow last
    $can create line [expr $x/2+$oydraw] [expr $y/2-$ozdraw] [expr $x-$xmarge+10] \
	    [expr $y/2-$ozdraw] -arrow last
    $can create oval [expr $x/2-3] [expr $y/2-3] [expr $x/2+3] [expr $y/2+3] -fill red
    $can create text [expr $x/2+10+$oydraw] [expr $ymarge+10] -text Z
    $can create text [expr $x-$xmarge-10] [expr $y/2-10-$ozdraw] -text Y
    $can create text [expr $x/2-3] [expr $y/2+10] -text G
    set iyy [format %.6g [expr (1.0/3.0)*($topwidth*pow($height-$cz,3)+$bottomwidth*pow($cz,3) \
	    -($topwidth-$thick)*pow($height-$cz-$wingsthick,3) \
	    -($bottomwidth-$thick)*pow($cz-$wingsthick,3))+pow($oz,2)*$area]]
    set izz [format %.6g [expr (1.0/12.0)*($wingsthick*pow($topwidth,3)+($height-2*$wingsthick)*pow($thick,3)+ \
	    $wingsthick*pow($bottomwidth,3))+pow($oy,2)*$area]]
    set iyz [format %.6g [expr abs($oz)*abs($oy)*$area]]
    set j [format %.6g [ expr 0.333*(($topwidth+$bottomwidth)*pow($wingsthick,3)+($height-2*$wingsthick)*pow($thick,3))]]
   
}    
proc sections::accepts { GDN STRUCT } {
    variable area 
    variable iyy
    variable izz
    variable iyz
    variable young
    variable mass
    variable shear
    variable matname
    variable units
    variable j
    variable can
    variable radius
    variable thickness
    variable width
    variable topwidth
    variable height
    variable topwidth
    variable height
    variable sectiontype
     
    if { [errorcntrl [list 7 "Young Modulus" $young "Shear Modulus" $shear Density $mass]] } { 
	return 
    }
    if { $area == "NaN" } {
	WarnWin " Please check the coordinates of the centroid." .t
	return
    }
    if { $area == 0.0 || $area == ""} {
	WarnWin "Don't forget to press Calculate button before" .t
    } else {
	DWLocalSetValue $GDN $STRUCT  "Area" [format %.6g $area]
	DWLocalSetValue $GDN $STRUCT  "Inertia_y" [format %.6g $iyy]
	DWLocalSetValue $GDN $STRUCT  "Inertia_z" [format %.6g $izz]
	DWLocalSetValue $GDN $STRUCT  "J" [format %.6g $j]
	DWLocalSetValue $GDN $STRUCT  "E" [format %.6g $young]
	DWLocalSetValue $GDN $STRUCT  "G" [format %.6g $shear]
	DWLocalSetValue $GDN $STRUCT  "Specific_weight" [format %.6g $mass]

	set ipos [lsearch [list N-m-kg N-cm-kg N-mm-kg Kp-cm-utm] $units]
	set fac_2mm [lindex [list 1e3 10 1 10] $ipos]
	set units_out [lindex [list N-m-kg N-cm-kg N-mm-kg Kp-cm-utm] $ipos]
	DWLocalSetValue $GDN $STRUCT  "Units" $units_out

	switch $sectiontype {
	    "Rectangular Solid" {
		set width_mm [expr {int($width*$fac_2mm)}]
		set height_mm [expr {int($height*$fac_2mm)}]
		set Name "R-${width_mm}-$height_mm" }
	    "Trapezoidal Solid" { set Name "-" }
	    "Circular Solid" {
		set diameter_mm [expr {int(2*$radius*$fac_2mm)}]
		set Name "O-$diameter_mm"
	    }
	    "Tube" {
		set diameter_mm [expr {int(2*$radius*$fac_2mm)}]
		set th_mm [expr {int($thickness*$fac_2mm)}]
		set Name "TUBE-$diameter_mm-$th_mm"
	    }
	    "SemiCircular Solid" { set Name "-" }
	    "Double T" {
		set topwidth_mm [expr {int($topwidth*$fac_2mm)}]
		set height_mm [expr {int($height*$fac_2mm)}]
		set Name "T-$topwidth_mm-$height_mm"
	    }
	    default { set Name "-" }    
	}
	DWLocalSetValue $GDN $STRUCT Name $Name 
	
	$can delete all
	set area 0.0
	set izz 0.0
	set iyy 0.0
	set iyz 0.0
	set j 0.0
	destroy .t
    }
}
proc sections::cancel { GDN STRUCT } {
    variable area 
    variable iyy
    variable izz
    variable iyz
    variable can 
    variable young
    variable mass
    variable shear
    variable matname
    variable units
    variable j
   
    $can delete all
    set area 0.0
    set izz 0.0
    set iyy 0.0
    set iyz 0.0
    set j 0.0
    #set young 0.0
    #set shear 0.0
    #set matname ""
    #set mass 0.0
    #set untis N-m-kg    

    destroy .t
    
}
proc sections::errorcntrl { values } {
    set message ""

    array set prop [ lrange $values 1 end]
    foreach elem [array names prop] {
	if { [string is space $prop($elem) ] } {
	    append message " $elem cannot be left blank.\n"
	} elseif { ! [string is double -strict $prop($elem) ] } {
	    append message " \"$prop($elem)\" is not a valid input for $elem. \n" 
	}
    }
    if  { $message != "" } {
	WarnWin $message .t
	return 1
    }
    return 0
}

proc sections::matproperties { } {

    variable matname
    variable young
    variable shear
    variable mass
    variable units 
    variable unitshow
    variable unitmass

    set matprop(steel) [list 2.1e11 8.1e10 76900]
    set matprop(titanium) [list 1.1e11 4.6e10 42112]
    set matprop(stainless) [list 2.0e11 7.9e10 53234]
    
    if { $matname != "" } {
	switch  $units {
	    "N-m-kg" {
		foreach "unitshow unitmass" "N/m\u00b2 N/m\u00b3" break
		set young [format %#8.3g [lindex $matprop($matname) 0]]
		set shear  [format %#8.3g [lindex $matprop($matname) 1]]
		set mass  [format %#8.3g [lindex $matprop($matname) 2]]           
	    }
	    "N-cm" {
		foreach "unitshow unitmass" "N/cm\u00b2 N/cm\u00b3" break
		set young  [format %#8.3g [expr 1.0e-4*[lindex $matprop($matname) 0]]]
		set shear  [format %#8.3g [expr 1.0e-4*[lindex $matprop($matname) 1]]]
		set mass  [format %#8.3g [expr 1.0e-6*[lindex $matprop($matname) 2]]]
	    }
	    "N-mm-kg" {
		foreach "unitshow unitmass" "N/mm\u00b2 N/mm\u00b3" break
		set young [format %#8.3g [expr 1.0e-6*[lindex $matprop($matname) 0]]]
		set shear [format %#8.3g [expr 1.0e-6*[lindex $matprop($matname) 1]]]
		set mass [format %#8.3g [expr 1.0e-9*[lindex $matprop($matname) 2]]]
	    }
	    "Kp-cm-utm" {
		foreach "unitshow unitmass" "Kp/cm\u00b2 Kp/cm\u00b3" break
		set young [format %#8.3g [expr 1.0e-4*[lindex $matprop($matname) 0]/9.81]]
		set shear [format %#8.3g [expr 1.0e-4*[lindex $matprop($matname) 1]/9.81]]
		set mass [format %#8.3g [expr 1.0e-6*[lindex $matprop($matname) 2]/9.81]]
	    }
	    default { }
	}
    }
}
proc sections::linkenrtyscale { type } { 

    variable yscale
    variable zscale
    variable area
    variable izz
    variable iyy
    variable iyz
    variable oz
    variable oy
    variable j

    switch $type {
	 "z" { 
	     if { [string is double -strict $oz ] } { 
	    sections::refresh
	    } else {
		set area "NaN"
		set izz "NaN"
		set iyy "NaN"
		set iyz "NaN"
		set j "NaN"
	    }
	}
	"y" {
	    if { [string is double -strict $oy ] } { 
	    sections::refresh
	    } else {
		set area "NaN"
		set izz "NaN"
		set iyy "NaN"
		set iyz "NaN"
		set j "NaN"
	    }
	}
    }
} 

proc sections::increasescale { type } {
    variable yscale
    variable zscale
    
    switch $type {
	"y" {
	    array set param [list to [expr 2*[$yscale cget -to]] \
		    from [expr 2*[$yscale cget -from]] \
		    tickinterval  [expr 2*[$yscale cget -tickinterval]] \
		    resolution  [expr 2*[$yscale cget -resolution]]]
	    
	    $yscale configure -to $param(to) -from $param(from) -tickinterval $param(tickinterval) \
		-resolution $param(resolution) 
	}
	"z" { 
	    array set param [list to [expr 2*[$zscale cget -to]] \
		    from [expr 2*[$zscale cget -from]] \
		    tickinterval  [expr 2*[$zscale cget -tickinterval]] \
		    resolution  [expr 2*[$zscale cget -resolution]]]
	    
	    $zscale configure -to $param(to) -from $param(from) -tickinterval $param(tickinterval) \
		-resolution $param(resolution) 
	}
    }
}
proc sections::decreasescale { type } {
    variable yscale
    variable zscale
    
    switch $type {
	"y" {
	    array set param [list to [expr [$yscale cget -to]/2] \
		    from [expr [$yscale cget -from]/2] \
		    tickinterval  [expr [$yscale cget -tickinterval]/2] \
		    resolution  [expr [$yscale cget -resolution]/2]]
	    
	    $yscale configure -to $param(to) -from $param(from) -tickinterval $param(tickinterval) \
		-resolution $param(resolution) 
	}
	"z" { 
	    array set param [list to [expr [$zscale cget -to]/2] \
		    from [expr [$zscale cget -from]/2] \
		    tickinterval  [expr [$zscale cget -tickinterval]/2] \
		    resolution  [expr [$zscale cget -resolution]/2]]
	    
	    $zscale configure -to $param(to) -from $param(from) -tickinterval $param(tickinterval) \
		-resolution $param(resolution) 
	}
    }
}